@@ -1,11 +1,11 @@
-#$Id: HP34401A.pm 529 2006-10-07 20:31:17Z schroeer $
+#$Id: HP34401A.pm 613 2010-04-14 20:40:41Z schroeer $
 
 package Lab::Instrument::HP34401A;
 
 use strict;
 use Lab::Instrument;
 
-our $VERSION = sprintf("0.%04d", q$Revision$ =~ / (\d+) /);
+our $VERSION = sprintf("0.%04d", q$Revision: 613 $ =~ / (\d+) /);
 
 sub new {
     my $proto = shift;
@@ -30,6 +30,18 @@ sub read_voltage_dc {
     return $value;
 }
 
+sub read_resistance {
+    my $self=shift;
+    my ($range,$resolution)=@_;
+    
+    $range="DEF" unless (defined $range);
+    $resolution="DEF" unless (defined $resolution);
+    
+    my $cmd=sprintf("MEASure:RESistance? %s,%s",$range,$resolution);
+    my ($value)=split "\n",$self->{vi}->Query($cmd);
+    return $value;
+}
+
 sub read_voltage_ac {
     my $self=shift;
     my ($range,$resolution)=@_;
@@ -112,6 +124,82 @@ sub reset {
     $self->{vi}->Write("*RST");
 }
 
+sub config_voltage {
+    my $self=shift;
+    my ($digits, $range, $counts)=@_;
+
+    #set input resistance to >10 GOhm for the three highest resolution values 
+    $self->{vi}->Write("INPut:IMPedance:AUTO ON");
+
+    $digits = int($digits);
+    $digits = 4 if $digits < 4;
+    $digits = 6 if $digits > 6;
+ 
+    if ($range < 0.1) {
+      $range = 0.1;
+    }
+    elsif ($range < 1) {
+      $range = 1;
+    }
+    elsif ($range < 10) {
+      $range = 10;
+    }
+    elsif ($range < 100) {
+      $range = 100;
+    }
+    else{
+      $range = 1000;
+    }
+
+    my $resolution = (10**(-$digits))*$range;
+    $self->{vi}->Write("CONF:VOLT:DC $range,$resolution");
+
+
+    # calculate integration time, set it and prepare for output
+ 
+    my $inttime = 0;
+
+    if ($digits ==4) {
+      $inttime = 0.4;
+      $self->{vi}->Write("VOLT:NPLC 0.02");
+    }
+    elsif ($digits ==5) {
+      $inttime = 4;
+      $self->{vi}->Write("VOLT:NPLC 0.2");
+    }
+    elsif ($digits ==6) {
+      $inttime = 200;
+      $self->{vi}->Write("VOLT:NPLC 10");
+      $self->{vi}->Write("ZERO:AUTO OFF");
+    }
+
+    my $retval = $inttime." ms";
+
+
+    # triggering
+    $self->{vi}->Write("TRIGger:SOURce BUS");
+    $self->{vi}->Write("SAMPle:COUNt $counts");
+    $self->{vi}->Write("TRIGger:DELay MIN");
+    $self->{vi}->Write("TRIGger:DELay:AUTO OFF");
+
+    return $retval;
+}
+
+sub read_with_trigger_voltage_dc {
+    my $self=shift;
+
+    $self->{vi}->Write("INIT");
+    $self->{vi}->Write("*TRG");
+    my $value = $self->{vi}->Query("FETCh?");
+
+    chomp $value;
+
+    my @valarray = split(",",$value);
+
+    return @valarray;
+}
+
+
 sub scroll_message {
     use Time::HiRes (qw/usleep/);
     my $self=shift;
@@ -128,6 +216,13 @@ sub id {
     $self->{vi}->Query('*IDN?');
 }
 
+sub read_value {
+    my $self=shift;
+    my $value=$self->{vi}->Query('READ?');
+    chomp $value;
+    return $value;
+}
+
 1;
 
 =head1 NAME
@@ -144,7 +239,8 @@ Lab::Instrument::HP34401A - HP/Agilent 34401A digital multimeter
 =head1 DESCRIPTION
 
 The Lab::Instrument::HP34401A class implements an interface to the 34401A digital multimeter by
-Agilent (formerly HP).
+Agilent (formerly HP). This module can also be used to address the newer 34410A and 34411A multimeters,
+but doesn't include new functions. Use the Lab::Instrument::HP34411A class for full functionality.
 
 =head1 CONSTRUCTOR
 
@@ -179,7 +275,7 @@ The best resolution is 100nV: C<$range=0.1>; C<$resolution=0.000001>.
 
 Preset and make an ac voltage measurement with the specified range
 and resolution. For ac measurements, resolution is actually fixed
-at 6 digits. The resolution parameter only affects the front-panel display.
+at 6 1/2 digits. The resolution parameter only affects the front-panel display.
 
 =head2 read_current_dc
 
@@ -194,7 +290,28 @@ and resolution.
 
 Preset and make an ac current measurement with the specified range
 and resolution. For ac measurements, resolution is actually fixed
-at 6 digits. The resolution parameter only affects the front-panel display.
+at 6 1/2 digits. The resolution parameter only affects the front-panel display.
+
+=head2 read_resistance
+
+    $datum=$hp->read_resistance($range,$resolution);
+
+Preset and measure resistance with specified range and resolution.
+
+=head2 config_voltage
+
+    $inttime=$hp->config_voltage($digits,$range,$count);
+
+Configures device for measurement with specified number of digits (4 to 6), voltage range and number of data
+points. Afterwards, data can be taken by triggering the multimeter, resulting in faster measurements than using
+read_voltage_xx.
+Returns string with integration time resulting from number of digits.
+
+=head2 read_with_trigger_voltage_dc
+
+    @array = $hp->read_with_trigger_voltage_dc()
+
+Take data points as configured with config_voltage(). returns an array.
 
 =head2 display_on
 
@@ -268,10 +385,11 @@ probably many
 
 =head1 AUTHOR/COPYRIGHT
 
-This is $Id: HP34401A.pm 529 2006-10-07 20:31:17Z schroeer $
+This is $Id: HP34401A.pm 613 2010-04-14 20:40:41Z schroeer $
 
-Copyright 2004-2006 Daniel Schrer (L<http://www.danielschroeer.de>)
+Copyright 2004-2006 Daniel Schröer (L<http://www.danielschroeer.de/>), 2009-2010 Daniela Taubert
 
-This library is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
+This library is free software; you can redistribute it and/or modify it under the same
+terms as Perl itself.
 
 =cut
